<?php

$xirca_config = array();

function xirca_user() {
	global $xirca_config;
	if ( isset($xirca_config["user"]) )
		return $xirca_config["user"];
	if ( isset($xirca_config["login"]) )
		return $xirca_config["login"];
	return FALSE;
}

function xirca_set_config($type, $value) {
	global $xirca_config;
	$xirca_config[$type] = $value;
}

function xirca_call($command, $user = "", $config = "") {
        return xirca_parse_response(xirca_call_raw($command, $user, $config));
}

function xirca_call_raw($command, $user = "", $config = "") {
	global $xirca_config;
	$url = $xirca_config["url"];
	$args = array();

	if ( isset($xirca_config["login"]) )
		$args["s_login"] = $xirca_config["login"];
	if ( isset($xirca_config["password"]) )
		$args["s_pw"] = $xirca_config["password"];
	if ( isset($xirca_config["user"]) )
		$args["s_user"] = $xirca_config["user"];
	if ( isset($xirca_config["entity"]) )
		$args["s_entity"] = $xirca_config["entity"];

	if (isset($_SESSION['AUTH']['SUBUSER'])) {
		$args["s_user"] .= " ".$_SESSION['AUTH']['SUBUSER'];
	}

	if ( strlen($user) )
		$args["s_user"] .= " ".$user;

	$args["s_command"] = xirca_encode_command($command);

	$curl = curl_init($url);
	if ( $curl === FALSE ) {
		return "[RESPONSE]\nCODE=423\nAPI access error: curl_init failed\nEOF\n";
	}

	$postfields = array();
	foreach ( $args as $key => $value ) {
		$postfields[] = urlencode($key)."=".urlencode($value);
	}
	$postfields = implode('&', $postfields);

	curl_setopt( $curl, CURLOPT_POST, 1 );
	curl_setopt( $curl, CURLOPT_POSTFIELDS, $postfields );
	curl_setopt( $curl, CURLOPT_HEADER, 0 );
	curl_setopt( $curl, CURLOPT_RETURNTRANSFER , 1 );

	$response = curl_exec($curl);
	return $response;
}

function xirca_call_retry($retries=1, $command, $user = "", $config = "") {
    for ($i=1;$i<=$retries;$i++) {
        $response = xirca_parse_response(xirca_call_raw($command, $user, $config));
        if (substr($response["CODE"],0,1) != "4") {
            return $response;
        }
    }
}

function xirca_encode_command( $commandarray ) {

    if (!is_array($commandarray)) return $commandarray;

    $command = "";

    foreach ( $commandarray as $k => $v ) {
        if ( is_array($v) ) {
	    $v = xirca_encode_command($v);
            $l = explode("\n", trim($v));
            foreach ( $l as $line ) {
                $command .= "$k$line\n";
	    }
        }
        else {
            $v = preg_replace( "/\r|\n/", "", $v );
            $command .= "$k=$v\n";
        }
    }

    return $command;
}


function xirca_parse_response ( $response ) {

    if (is_array($response)) return $response;

    if (!$response) return array( "CODE" => "423", "DESCRIPTION" => "Empty response from API");
    
    $hash = array("PROPERTY" => array());
    
    $rlist = explode( "\n", $response );

    foreach ( $rlist as $item ) {
        if ( preg_match("/^([^\=]*[^\t\= ])[\t ]*=[\t ]*(.*)$/", $item, $m) ) {
            $attr = $m[1];
            $value = $m[2];
            $value = preg_replace( "/[\t ]*$/", "", $value );
            if ( preg_match( "/^property\[([^\]]*)\]/i", $attr, $m) ) {
                $prop = strtoupper($m[1]);
                $prop = preg_replace( "/\s/", "", $prop );
                if ( in_array($prop, array_keys($hash["PROPERTY"])) ) {
                    array_push($hash["PROPERTY"][$prop], $value);
                }
                else {
                     $hash["PROPERTY"][$prop] = array($value);
                }
            }
            else {
                $hash[strtoupper($attr)] = $value;
            }
        }
    }
    return $hash;
}


?>
